/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs;

public class PWLCF {
	public Segment left;
	public Segment right;
	public int x;
	public int y;
	
	public PWLCF(){
		left = new Segment(Integer.MIN_VALUE,Integer.MAX_VALUE);
		right = new Segment(Integer.MIN_VALUE,Integer.MAX_VALUE);
		left.next = new Segment(Integer.MAX_VALUE,Integer.MAX_VALUE);
		right.next = new Segment(Integer.MAX_VALUE,Integer.MAX_VALUE);
		x = 0;
		y = 0;
	}
	public void close(){
		left = left.next;
		right = right.next;
	}
	public String toString(){
		String res = "";
		res+=("x = "+x)+"\n";
		res+=("y = "+y)+"\n";
		res+=("right = ");
		Segment tmp = right;
		while(tmp!=null){
			res+=(tmp+"; ");
			tmp = tmp.next;
		}
		res+="\n";
		res+=("left = ");
		tmp = left;
		while(tmp!=null){
			res+=(tmp+"; ");
			tmp = tmp.next;
		}
		res+="\n";
		return res;
	}

	public Segment addSegment(int sslope, int slength,Segment cur){
		Segment prev = null;
		while(sslope > cur.slope){
			prev = cur;
			cur = cur.next;
		}
		if(sslope == cur.slope){
			cur.length += slength;
		}else{

			Segment s = new Segment(sslope, slength);
			prev.next = s;
			s.next = cur;
			cur = s;
		}
		return cur;
	}
	
	public void moveRootRight(int dist) {
		x += dist;
		while(dist > 0){
			int l = Math.min(dist,right.length);
			y += l * right.slope;
			if(right.slope == - left.slope){
				left.length += l;
				right.length -= l;
				if(right.length==0){
					right = right.next;
				}
			}else{
				Segment cur = null;
				if(right.length == l){
					cur = right;
					cur.slope = -right.slope;
					right = right.next;
				}else{
					cur = new Segment(-right.slope,l);
					right.length -= l;
				}
				cur.next = left;
				left = cur;
			}
			dist -= l;
		}
	}
	public void moveRootLeft(int dist) {
		x -= dist;
		while(dist > 0){
			int l = Math.min(dist,left.length);
			y += l * left.slope;
			if(right.slope == - left.slope){
				right.length += l;
				left.length -= l;
				if(left.length==0){
					left = left.next;
				}
			}else{
				Segment cur = null;
				if(left.length == l){
					cur = left;
					cur.slope = -left.slope;
					left = left.next;
				}else{
					cur = new Segment(-left.slope,l);
					left.length -= l;
				}
				cur.next = right;
				right = cur;
			}
			dist -= l;
		}
	}
}
