/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.examples;


import convexPairs.Parameters;

public class SumAloneC2 implements Parameters {
	private int n;
	private int[] lx;
	private int[] ux;
	
	public SumAloneC2(int n, int[] ub, int[] lb){
		this.n = n;
		this.ux = ub;
		this.lx = lb;
	}
	
	public String toString() {	
		return "Sum alone as C2";
	}
	public int getN() {
		return n;
	}
	public int evaluateF(int i, int x) {
		return 0;
	}
	public int evaluateG(int i, int x) {
		return x;
	}
	public int getArgminF(int i) {
		return lx[i];//any value would fit 
	}
	public int getDeltaP(int i, int Wi) {
		return Wi < ux[i]? 0: Integer.MAX_VALUE;
	}
	public int getBPP(int i, int Wi) {
		return ux[i];
	}
	public int getDeltaN(int i, int Wi) {
		return Wi > lx[i]? 0 : Integer.MAX_VALUE;
	}
	public int getBPN(int i, int Wi) {
		return lx[i];
	}
	public boolean useGeneralCase(int i) {
		return false;
	}
	public void enforceGLessThanOrEqual(int i, int v) {
		// x <= v
	}
	public void enforceGGreaterThanOrEqual(int i, int v) {
		// x >= v
	}
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int u) {
		// x=v -> 0 <= u
	}
}
