/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.inoscar;


import convexPairs.ConvexPairOfSum;
import convexPairs.inoscar.FailureException;
import convexPairs.inoscar.ParametersWithOscar;
import oscar.cp.core.*;

public class GenericPropagator extends Constraint {
	private CPVarInt[] X;
	private CPVarInt S1;
	private CPVarInt S2;
	private CPVarInt S3;
	private int s1;
	private int s2;
	private int s3;
	private ParametersWithOscar h;
	private ConvexPairOfSum na;
	public GenericPropagator(CPVarInt[] X, ParametersWithOscar h, CPVarInt S1, int s1, CPVarInt S2, int s2, CPVarInt S3, int s3, String name, boolean sort){
		super(X[0].s(),name);
		this.X = X;
		this.S1 = S1;
		this.S2 = S2;
		this.S3 = S3;
		this.s1 = s1;
		this.s2 = s2;
		this.s3 = s3;
		this.h = h;
		na = new ConvexPairOfSum(h);
		na.sort = sort;
	}
	
	protected CPOutcome setup(CPPropagStrength l){
        for (int i = 0; i < X.length; i++) {
            if (!X[i].isBound()){
            	if(h.useGeneralCase(i))
            		X[i].callPropagateWhenDomainChanges(this);
            	else
            		X[i].callPropagateWhenBoundsChange(this);
            }
                
        }
        if(S1!=null)S1.callPropagateWhenMaxChanges(this);
        if(S2!=null)S2.callPropagateWhenMinChanges(this);
        if(S3!=null)S3.callPropagateWhenMaxChanges(this);
        return propagate();
	}
	
	protected CPOutcome propagate(){
		h.updateDomains();
		int LB = na.computeLB(S2!=null?S2.getMin():s2,S3!=null?S3.getMax():s3);
		if(S1!=null){
			if(S1.updateMin(LB)==CPOutcome.Failure) return CPOutcome.Failure;
		}else{
			if(LB > s1) return CPOutcome.Failure;
		}
		if(S1!=null) s1 = S1.getMax();
		
		if(S2!=null && S2.updateMax(na.computeU_G(s1))==CPOutcome.Failure) return CPOutcome.Failure;
		if(S3!=null && S3.updateMin(na.computeL_G(s1))==CPOutcome.Failure) return CPOutcome.Failure;
		
		if(S2!=null) s2 = S2.getMin();
		if(S3!=null) s3 = S3.getMax();
		try{
			for(int i=0;i < X.length;i++){
				if(h.useGeneralCase(i)){
					na.generalCaseForward(i, s1, s2, s3);
					na.generalCaseBackward(i, s1, s2, s3);
				}else{
					na.specialCaseForward(i, s1, s2, s3);
					na.specialCaseBackward(i, s1, s2, s3);
				}
			}
		}catch(FailureException e){
			return CPOutcome.Failure;
		}
		return CPOutcome.Suspend;
	}
}
